#include <iostream>
#include <assert.h>
#include <vector>
#include <algorithm>
#include <stdio.h>
using namespace std;
typedef long long llong;

const int MAXN = 1000111;

int n;
int nums[MAXN];
vector<int> spots[MAXN];

struct event
{
    int x;
    int L, R, val;

    bool operator<(const event& other) const
    {
        return x < other.x;
    }
};

vector<event> events;
void badRectangle(int L1, int R1, int L2, int R2)
{
    event entry, exit;

    entry.x = L1;
    entry.L = L2;
    entry.R = R2;
    entry.val = 1;

    exit.x = R1 + 1;
    exit.L = L2;
    exit.R = R2;
    exit.val = -1;

    events.push_back(entry);
    events.push_back(exit);
}

struct Node
{
    int minVal = 0;
    int logVal = 0;
};

Node IT[4 * MAXN];
int LEAFOFFSET = 1;

void Refresh(int ver)
{
    if (IT[ver].logVal != 0)
    {
        IT[ver].minVal += IT[ver].logVal;

        if (ver <= LEAFOFFSET)
        {
            IT[2 * ver].logVal += IT[ver].logVal;
            IT[2 * ver + 1].logVal += IT[ver].logVal;
        }

        IT[ver].logVal = 0;
    }
}

void recUpd(int ver, int L, int R, int l, int r, int val)
{
    Refresh(ver);

    if (L > r || R < l)
        return;
    else if (L >= l && R <= r)
    {
        IT[ver].logVal = val;
        Refresh(ver);
    }
    else
    {
        recUpd(2 * ver, L, (L + R) / 2, l, r, val);
        recUpd(2 * ver + 1, (L + R) / 2 + 1, R, l, r, val);
        IT[ver].minVal = min(IT[2 * ver].minVal, IT[2 * ver + 1].minVal);
    }
}

void upd(int L, int R, int val)
{
    recUpd(1, 1, LEAFOFFSET + 1, L, R, val);
}

void recGet(int ver, int L, int R, int l, int r, vector<int>& result)
{
    Refresh(ver);
    if (IT[ver].minVal != 0)
        return;

    if (L > r || R < l)
        return;
    else if (L == R)
    {
        result.push_back(L);
    }
    else
    {
        recGet(2 * ver, L, (L + R) / 2, l, r, result);
        recGet(2 * ver + 1, (L + R) / 2 + 1, R, l, r, result);
    }

}

void getZeros(int L, int R, vector<int>& result)
{
    recGet(1, 1, LEAFOFFSET + 1, L, R, result);
}

pair<int, int> interesting[3 * MAXN];
int LEN = 0;
int leftMap[MAXN];
int rightMap[MAXN];

vector<int> valIdxs[MAXN];

struct PersistentNode
{
    int sum = 0;
    PersistentNode* s1 = nullptr;
    PersistentNode* s2 = nullptr;
};
const int MAXMEM = 3 * MAXN * 23;
PersistentNode MEM[MAXMEM];
int memCtr = 0;
inline PersistentNode* alloc()
{
    assert(memCtr < MAXMEM - 1);
    return &MEM[memCtr++];
}

PersistentNode* roots[3 * MAXN];
int PLEAFOFFSET = 1;

PersistentNode* incVal(PersistentNode* root, int idx)
{
    PersistentNode* cur = alloc();
    *cur = *root;
    PersistentNode* altRoot = cur;

    int L = 1, R = PLEAFOFFSET + 1;

    cur->sum++;
    //printf("Increase root clone %d\n", cur);
    while(L < R)
    {
        PersistentNode* kid = alloc();
        if (idx <= (L + R) / 2)
        {
            if (cur->s1)
                *kid = *(cur->s1);
            cur->s1 = kid;
            R = (L + R) / 2;
        }
        else
        {
            if (cur->s2)
                *kid = *(cur->s2);
            cur->s2 = kid;
            L = (L + R) / 2 + 1;
        }
        cur = kid;

        cur->sum++;
        //printf("Increase node %d\n", cur);
    }

    return altRoot;
}

int recSum(PersistentNode* root, int L, int R, int l, int r)
{
    //printf("Looking for %d - %d in %d - %d [ver %d]\n", l, r, L, R, root);
    if (!root)
        return 0;

    if (L > r || R < l)
        return 0;
    else if (L >= l && R <= r)
    {
        //printf("Taking in full sum of %d\n", root);
        return root->sum;
    }
    else
    {
        return recSum(root->s1, L, (L + R) / 2, l, r) + recSum(root->s2, (L + R) / 2 + 1, R, l, r);
    }
}

int getSum(int L, int R, PersistentNode* root)
{
    //printf("Calling for %d - %d\n", L, R);
    //printf("PLEAF OFFSET IS %d\n", PLEAFOFFSET);
    return recSum(root, 1, PLEAFOFFSET + 1, L, R);
}

void buildInteresting()
{
    PLEAFOFFSET = 1;
    while(PLEAFOFFSET <= LEN)
        PLEAFOFFSET *= 2;
    PLEAFOFFSET--;

    //printf("LEN = %d\n", LEN);
    //printf("PLEAFOFFSET = %d\n", PLEAFOFFSET);

    leftMap[1] = 1;
    rightMap[1] = 0;
    while(rightMap[1] + 1 <= LEN && interesting[ rightMap[1] + 1 ].first <= 1)
        rightMap[1]++;

    for (int i = 2; i <= n; i++)
    {
        leftMap[i] = leftMap[i - 1];
        while(leftMap[i] <= LEN && interesting[ leftMap[i] ].first < i)
            leftMap[i]++;

        rightMap[i] = rightMap[i - 1];
        while(rightMap[i] + 1 <= LEN && interesting[ rightMap[i] + 1 ].first <= i)
            rightMap[i]++;
    }

    for (int i = 1; i <= LEN; i++)
    {
        valIdxs[ interesting[i].second ].push_back(i);
    }

    //printf("Left; right = %d; %d\n", leftMap[492], rightMap[497]);

    PersistentNode* root = alloc();
    for (int i = 1; i <= n; i++)
    {
        for (auto idx : valIdxs[i])
        {
            //if (idx != 496 || i != 496)
            //    continue;

            //printf("Increasing %d for %d\n", idx, i);
            root = incVal(root, idx);
        }
        roots[i] = root;
    }
}

void init(int N, int Q, const std::vector<int> &input)
{
    assert(N < MAXN);
    assert(input.size() == N);
    n = N;
    for (int i = 0; i < n; i++)
    {
        nums[i + 1] = input[i];
    }

    for (int i = 1; i <= n; i++)
    {
        assert(nums[i] <= n);
        spots[ nums[i] ].push_back(i);
    }

    for (int i = 1; i <= n; i++)
    {
        for (int j = 0; j < spots[i].size(); j++)
        {
            int leftL = (j == 0) ? 1 : spots[i][j - 1] + 1;
            int leftR = spots[i][j];

            /// Too few
            if (i > 1)
            {
                int rightL = spots[i][j];
                int rightR;
                if (j + i - 1 >= spots[i].size())
                    rightR = n;
                else
                    rightR = spots[i][j + i - 1] - 1;
                badRectangle(leftL, leftR, rightL, rightR);
            }
            /// Too many
            if (j + i < spots[i].size())
            {
                int rightL = spots[i][j + i];
                int rightR = n;
                badRectangle(leftL, leftR, rightL, rightR);
            }
        }
    }

    LEAFOFFSET = 1;
    while(LEAFOFFSET < n)
        LEAFOFFSET *= 2;
    LEAFOFFSET--;

    sort(events.begin(), events.end());

    int curE = 0;
    for (int i = 1; i <= n; i++)
    {
        while(curE < events.size() && events[curE].x <= i)
        {
            upd(events[curE].L, events[curE].R, events[curE].val);
            curE++;
        }

        vector<int> rends;
        getZeros(i, n, rends);

        for (int r : rends)
        {
            LEN++;
            assert(LEN <= 3 * n);
            interesting[LEN] = {i, r};
        }
    }

    buildInteresting();
}

long long query(int l, int r)
{
	++ l; ++ r;
    if (LEN == 0)
        return 0;

    llong ans = 0;
    int L = leftMap[l];
    int R = rightMap[r];

    ans = getSum(L, R, roots[r]);

	return ans;
}

