#include<iostream>
#include<vector>
#include<fstream>
#include<chrono>
#include<random>
#include<cassert>

extern void init(int, int, const std::vector<int>&);
extern long long query(int, int);
static std::ofstream graderTimeFile;

static double totalGraderTime = 0;
static std::chrono::steady_clock::time_point startT;
static std::chrono::steady_clock::time_point endT;

static std::mt19937 rng;

static int random_int(int a, int b){
	int c = rng() % (b - a + 1) + a;
	assert(a <= c && c <= b);
	return c;
}

static void startTime(){
    startT = std::chrono::steady_clock::now();
}

static void writeTime(){
    graderTimeFile << "io-time: " << totalGraderTime << std::endl;
}

static void endTime(bool write = true){
    endT = std::chrono::steady_clock::now();

    double time = std::chrono::duration_cast<std::chrono::duration<double>>(endT - startT).count();

    totalGraderTime += time;

    if(write) writeTime();
}

int main(int argc, char* argv[]){
	graderTimeFile.open(argv[1]);
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr); std::cout.tie(nullptr);
	startTime();
	int n, q, seed; std::string qry_type; std::cin >> n >> q >> qry_type >> seed;
	rng.seed(seed);
	std::vector<int> input(n);
	for(int i = 0; i < n; i ++){
		std::cin >> input[i];
	}
	std::vector<std::pair<int, int>> queries;
	if(q == 1){
		// cout << "1 " << n << endl;
		queries.push_back({1, n});
	}
	else{
		if(qry_type == "sample"){
			queries.push_back({1, 4});
			queries.push_back({3, 7});
			queries.push_back({1, 7});
		}
		if(qry_type == "exhaust"){
			int len = n, lft = 1;
			for(int i = 0; i < q; i ++){
				// cout << lft << " " << lft + len - 1 << endl;
				queries.push_back({lft, lft + len - 1});
				if(lft + len - 1 == n){
					lft = 1;
					-- len;
				}
				else{
					++ lft;
				}
			}
		}
		if(qry_type == "random"){
			for(int i = 0; i < q; i ++){
				int l = random_int(1, n); // int l = rnd.next(1, n);
				int r = random_int(1, n); // int r = rnd.next(1, n);
				if(l > r) std::swap(l, r);
				// cout << l << " " << r << endl;
				queries.push_back({l, r});
			}
		}
		if(qry_type == "long"){
			for(int i = 0; i < q; i ++){
				int len = random_int(n / 2, n); // int len = rnd.next(n / 2, n);
				int l = random_int(1, n - len + 1);// int l = rnd.next(1, n - len + 1);
				queries.push_back({l, l + len - 1});
				// cout << l << " " << l + len - 1 << endl;
			}
		}
	}
	endTime();
	init(n, q, input);
	long long hash1 = 0, pw1 = 1; const long long base1 = 37, mod1 = 1e9 + 7;
	long long hash2 = 0, pw2 = 1; const long long base2 = 53, mod2 = 1e9 + 9;
	long long hash3 = 0, pw3 = 1; const long long base3 = 71, mod3 = 998244353;
	for(auto &[l, r] : queries){
		-- l; -- r;
		long long t = query(l, r);
		hash1 = (hash1 + t * pw1 % mod1) % mod1; pw1 = pw1 * base1 % mod1;
		hash2 = (hash2 + t * pw2 % mod2) % mod2; pw2 = pw2 * base2 % mod2;
		hash3 = (hash3 + t * pw3 % mod3) % mod3; pw3 = pw3 * base3 % mod3;
	}
	std::cout << hash1 << " " << hash2 << " " << hash3 << std::endl;
}
