#include <bits/stdc++.h>
// #include <coding_library/data_structures/wavelet_tree.hpp>
// #include <coding_library/data_structures/segment_tree_lazy.hpp>
// #include <coding_library/data_structures/monoids_lazy.hpp>

using namespace std;

template<
    class T, T (*merge)(T, T), T (*e)(), class G, T (*lazy_apply)(G, T),
    G (*lazy_merge)(G, G), G (*lazy_init)(T)>
class SegmentTreeLazy {
  private:
    int n, size, log_size;
    vector<T> tr;
    vector<G> lazy;

    void pull(int x) { tr[x] = merge(tr[2 * x], tr[2 * x + 1]); }

    void push_one(int x, G f) {
        tr[x] = lazy_apply(f, tr[x]);
        if(x < size) {
            lazy[x] = lazy_merge(f, lazy[x]);
        }
    }

    void push(int x) {
        push_one(2 * x, lazy[x]);
        push_one(2 * x + 1, lazy[x]);
        lazy[x] = lazy_init(tr[x]);
    }

    void push_all_down(int x) {
        for(int i = log_size; i >= 1; i--) {
            push(x >> i);
        }
    }

    void push_all_open_range(int l, int r) {
        for(int i = log_size; i >= 1; i--) {
            if(((l >> i) << i) != l) {
                push(l >> i);
            }
            if(((r >> i) << i) != r) {
                push((r - 1) >> i);
            }
        }
    }

  public:
    SegmentTreeLazy() { init(vector<T>()); }
    SegmentTreeLazy(int _n) { init(vector<T>(_n, e())); }
    SegmentTreeLazy(const vector<T>& _a) { init(_a); }

    void init(const vector<T>& _a) {
        n = _a.size();
        size = 1;
        log_size = 0;
        while(size < n) {
            size <<= 1;
            log_size++;
        }

        tr.assign(2 * size, e());
        lazy.resize(size);
        for(int i = 0; i < n; i++) {
            tr[size + i] = _a[i];
        }
        for(int i = size - 1; i > 0; i--) {
            pull(i);
            lazy[i] = lazy_init(tr[i]);
        }
    }

    void update(int pos, T val) {
        pos += size;
        push_all_down(pos);
        tr[pos] = val;
        for(pos >>= 1; pos > 0; pos >>= 1) {
            pull(pos);
        }
    }

    void apply_lazy(int l, int r, G f) {
        if(l > r) {
            return;
        }

        l += size, r += size + 1;
        push_all_open_range(l, r);
        for(int l0 = l, r0 = r; l0 < r0; l0 >>= 1, r0 >>= 1) {
            if(l0 & 1) {
                push_one(l0++, f);
            }
            if(r0 & 1) {
                push_one(--r0, f);
            }
        }

        for(int i = 1; i <= log_size; i++) {
            if(((l >> i) << i) != l) {
                pull(l >> i);
            }
            if(((r >> i) << i) != r) {
                pull((r - 1) >> i);
            }
        }
    }

    T get_pos(int pos) {
        pos += size;
        push_all_down(pos);
        return tr[pos];
    }

    T query(int l, int r) {
        T ansl = e(), ansr = e();
        l += size, r += size + 1;
        push_all_open_range(l, r);

        for(; l < r; l >>= 1, r >>= 1) {
            if(l & 1) {
                ansl = merge(ansl, tr[l++]);
            }
            if(r & 1) {
                ansr = merge(tr[--r], ansr);
            }
        }
        return merge(ansl, ansr);
    }

    T query_all() { return tr[1]; }

    void print(int l, int r) {
        for(int i = l; i <= r; i++) {
            auto data = get_pos(i);
            cout << "[ " << data.val << ", " << data.pos << " ] , ";
        }
        cout << endl;
    }

    template<bool (*f)(T)>
    int max_right(int l) const {
        return max_right(l, [](T x) { return f(x); });
    }
    template<class F>
    int max_right(int l, F f) const {
        if(l == n) {
            return n;
        }

        l += size;
        push_all_down(l);
        T sm = e();
        do {
            while(l % 2 == 0) {
                l >>= 1;
            }
            if(!f(merge(sm, tr[l]))) {
                while(l < size) {
                    push(l);
                    l = (2 * l);
                    if(f(merge(sm, tr[l]))) {
                        sm = merge(sm, tr[l]);
                        l++;
                    }
                }
                return l - size;
            }
            sm = merge(sm, tr[l]);
            l++;
        } while((l & -l) != l);
        return n;
    }

    template<bool (*f)(T)>
    int min_left(int r) const {
        return min_left(r, [](T x) { return f(x); });
    }
    template<class F>
    int min_left(int r, F f) const {
        if(r == -1) {
            return 0;
        }

        r += size + 1;
        push_all_down(r - 1);
        T sm = e();
        do {
            r--;
            while(r > 1 && (r % 2)) {
                r >>= 1;
            }
            if(!f(merge(tr[r], sm))) {
                while(r < size) {
                    push(r);
                    r = (2 * r + 1);
                    if(f(merge(tr[r], sm))) {
                        sm = merge(tr[r], sm);
                        r--;
                    }
                }
                return r + 1 - size;
            }
            sm = merge(tr[r], sm);
        } while((r & -r) != r);
        return 0;
    }
};

class WaveletTree {
  private:
    int lo{1}, hi{0};
    WaveletTree *left{nullptr}, *right{nullptr};
    vector<int> b;
    int psz{0};

  public:
    WaveletTree() = default;

    void init(const vector<int>& arr, int x, int y) {
        lo = x;
        hi = y;
        if(lo == hi || arr.empty()) {
            return;
        }
        int mid = (lo + hi) >> 1;
        auto f = [mid](int x) { return x <= mid; };
        b.resize(arr.size() + 1);
        b[0] = 0;
        for(size_t i = 0; i < arr.size(); ++i) {
            b[i + 1] = b[i] + f(arr[i]);
        }
        psz = arr.size();

        // Create a mutable copy of the array
        vector<int> arr_copy(arr);
        auto pivot = stable_partition(arr_copy.begin(), arr_copy.end(), f);

        left = new WaveletTree();
        right = new WaveletTree();
        left->init(vector<int>(arr_copy.begin(), pivot), lo, mid);
        right->init(vector<int>(pivot, arr_copy.end()), mid + 1, hi);
    }

    int kth(int l, int r, int k) const {
        if(l > r) {
            return 0;
        }
        if(lo == hi) {
            return lo;
        }
        int inLeft = b[r] - b[l - 1], lb = b[l - 1], rb = b[r];
        if(k <= inLeft) {
            return left->kth(lb + 1, rb, k);
        }
        return right->kth(l - lb, r - rb, k - inLeft);
    }

    int LTE(int l, int r, int k) const {
        if(l > r || k < lo) {
            return 0;
        }
        if(hi <= k) {
            return r - l + 1;
        }
        int lb = b[l - 1], rb = b[r];
        return left->LTE(lb + 1, rb, k) + right->LTE(l - lb, r - rb, k);
    }

    int count(int l, int r, int k) const {
        if(l > r || k < lo || k > hi) {
            return 0;
        }
        if(lo == hi) {
            return r - l + 1;
        }
        int lb = b[l - 1], rb = b[r];
        int mid = (lo + hi) >> 1;
        if(k <= mid) {
            return left->count(lb + 1, rb, k);
        }
        return right->count(l - lb, r - rb, k);
    }
};

template<class T>
class MonoidMin {
  public:
    struct MinVal {
        T val;
        int pos;
    };

    struct AddLazy {
        T add_val;
    };

    static MinVal merge(MinVal a, MinVal b) {
        if(a.val == b.val) {
            return {a.val, min(a.pos, b.pos)};
        } else if(a.val < b.val) {
            return a;
        } else {
            return b;
        }
    }

    static MinVal e() { return {numeric_limits<T>::max(), -1}; }

    static MinVal lazy_apply(AddLazy f, MinVal x) {
        return {x.val + f.add_val, x.pos};
    }

    static AddLazy lazy_merge(AddLazy a, AddLazy b) {
        return {a.add_val + b.add_val};
    }

    static AddLazy lazy_init(MinVal _) { return {0}; }
};

vector<int> range_lefts, range_rights;
WaveletTree* wvlt;

SegmentTreeLazy<
    MonoidMin<int>::MinVal, MonoidMin<int>::merge, MonoidMin<int>::e,
    MonoidMin<int>::AddLazy, MonoidMin<int>::lazy_apply,
    MonoidMin<int>::lazy_merge, MonoidMin<int>::lazy_init>
    st;

void init(int n, int q, const vector<int>& vals) {
    vector<MonoidMin<int>::MinVal> init_vals;
    for(int i = 0; i < n; i++) {
        init_vals.push_back({0, i});
    }
    st.init(init_vals);

    vector<vector<int>> pos(n + 1);

    function<pair<int, int>(int)> get_range_for_val = [&](int x) {
        if(x > n || pos[x].size() < x) {
            return pair<int, int>{-1, -1};
        }

        int r = pos[x][(int)pos[x].size() - x];
        int i = (int)pos[x].size() - x - 1;

        if(i < 0) {
            return pair<int, int>{0, r};
        } else {
            return pair<int, int>{pos[x][i] + 1, r};
        }
    };

    for(int i = 0; i < n; i++) {
        int x = vals[i];
        st.apply_lazy(0, i, {1});
        if(x > n || x <= 0) {
            continue;
        }

        if(!pos[x].empty()) {
            st.apply_lazy(0, pos[x].back(), {-1});
        }

        auto [prvl, prvr] = get_range_for_val(x);
        if(prvl != -1) {
            st.apply_lazy(prvl, prvr, {1});
        }

        pos[x].push_back(i);
        auto [l, r] = get_range_for_val(x);
        if(l != -1) {
            st.apply_lazy(l, r, {-1});
        }

        // cout << "After i: " << i << endl;
        // st.print(0, i);

        int start = 0;
        while(start <= i) {
            MonoidMin<int>::MinVal min_val = st.query(start, i);
            if(min_val.val != 0) {
                break;
            }

            int min_pos = min_val.pos;
            // cout << "Adding range: " << min_pos << " " << i << endl;
            range_lefts.push_back(min_pos);
            range_rights.push_back(i);

            start = min_pos + 1;
        }
    }

    wvlt = new WaveletTree();
    wvlt->init(range_lefts, 0, n);
}

long long query(int l, int r) {
    int ql = lower_bound(range_rights.begin(), range_rights.end(), l) -
             range_rights.begin();
    int qr = upper_bound(range_rights.begin(), range_rights.end(), r) -
             range_rights.begin() - 1;
    if(qr < ql) {
        return 0;
    }

    return (qr - ql + 1) - wvlt->LTE(ql + 1, qr + 1, l - 1);
}

