#include<bits/stdc++.h>
using namespace std;
 
const int MAXN = 1e6;
const int MAXQ = 5e5;
const int INF = 1e9;

struct KthSegmentTree{
	struct Node{
		int lazy = 0, cnt_min = 0, val_min = 0;
		Node() {}
		Node(int a, int b){ cnt_min = a; val_min = b; }
	} segmtree[4 * MAXN];
	int _n;
	KthSegmentTree(int _n) : _n(_n) {}
	void push(int node, int l, int r){
		segmtree[node].val_min += segmtree[node].lazy;
		if(l != r){
			segmtree[2 * node + 0].lazy += segmtree[node].lazy;
			segmtree[2 * node + 1].lazy += segmtree[node].lazy;
		}
		segmtree[node].lazy = 0;
	}

	Node mrg(const Node& a, const Node& b){
		if(a.val_min == b.val_min) return Node(a.cnt_min + b.cnt_min, a.val_min);
		if(a.val_min < b.val_min) return Node(a.cnt_min, a.val_min);
		return Node(b.cnt_min, b.val_min);
	}

	void update(int node, int l, int r, int ql, int qr, int val){
		if(ql > qr) return;
		push(node, l, r);
		if(qr < l || r < ql) return;
		if(ql <= l && r <= qr){
			segmtree[node].lazy += val;
			push(node, l, r);
		}
		else{
			int m = (l + r) / 2;
			push(node, l, r);
			update(2 * node + 0, l + 0, m, ql, qr, val);
			update(2 * node + 1, m + 1, r, ql, qr, val);
			segmtree[node] = mrg(segmtree[2 * node + 0], segmtree[2 * node + 1]);
		}
	}
	void update(int l, int r, int val){ /*cout << (val > 0 ? '+' : '-') << "[" << l << " " << r << "]" << endl;*/ update(1, 1, _n, l, r, val); }
	Node query(int node, int l, int r, int ql, int qr){
		if(qr < l || r < ql) return Node(0, INF);
		if(ql <= l && r <= qr){
			push(node, l, r);
			return segmtree[node];
		}
		else{
			int m = (l + r) / 2;
			push(node, l, r);
			return mrg(query(2 * node + 0, l + 0, m, ql, qr), query(2 * node + 1, m + 1, r, ql, qr));
		}
	}
	Node query(int l, int r){ return query(1, 1, _n, l, r); }
	void build(int node, int l, int r){
		segmtree[node].cnt_min = r - l + 1;
		if(l == r) return;
		int m = (l + r) / 2;
		build(2 * node + 0, l + 0, m);
		build(2 * node + 1, m + 1, r);
	}
	void build(){ build(1, 1, _n); }
	int descend(int node, int l, int r, int kth){
		// cout << "d" << node << " " << l << " " << r << " " << kth << endl;
		push(node, l, r);
		if(l == r) return l;
		int m = (l + r) / 2;
		push(2 * node + 0, l + 0, m);
		push(2 * node + 1, m + 1, r);
		int mymin = segmtree[node].val_min;
		int a = (segmtree[2 * node + 0].val_min == mymin ? segmtree[2 * node + 0].cnt_min : 0);
		int b = (segmtree[2 * node + 1].val_min == mymin ? segmtree[2 * node + 1].cnt_min : 0);
		if(kth <= a) return descend(2 * node + 0, l + 0, m, kth);
		else		 return descend(2 * node + 1, m + 1, r, kth - a);
	}
	int descend(int kth){ return descend(1, 1, _n, kth); }
};

struct PersistentTree{
	struct Node{
		Node *l, *r;
		int sum;
		Node() {}
		Node(int sum) : l(nullptr), r(nullptr), sum(sum) {}
		Node(Node* l, Node* r) : l(l), r(r){
			sum = 0;
			if(l) sum += l -> sum;
			if(r) sum += r -> sum;
		}
	};
	int _n;
	Node* roots[MAXN + 5];
	PersistentTree() {}
	PersistentTree(int _n) : _n(_n) {}
	Node* build(int l, int r){
		if(l == r) return new Node(0);
		int m = (l + r) / 2;
		return new Node(build(l, m + 0), build(m + 1, r));
	}
	void build(){ roots[0] = build(1, _n); }
	Node* update(Node* node, int l, int r, int x, int v){
		if(l == r) return new Node(node -> sum + v);
		int m = (l + r) / 2;
		if(l <= x && x <= m) return new Node(update(node -> l, l + 0, m, x, v), node -> r);
		else				 return new Node(node -> l, update(node -> r, m + 1, r, x, v));
	}
	void update(int old_ver, int new_ver, int x, int v){ roots[new_ver] = update(roots[old_ver], 1, _n, x, v); }
	int query(Node* node, int l, int r, int ql, int qr){
		if(qr < l || r < ql) return 0;
		else if(ql <= l && r <= qr) return node -> sum;
		else{
			int m = (l + r) / 2;
			return query(node -> l, l + 0, m, ql, qr) + 
				   query(node -> r, m + 1, r, ql, qr);
		}
	}
	int query(int version, int l, int r){ return query(roots[version], 1, _n, l, r); }
	void explore(Node* node, int l, int r){
		if(l == r) return;
		int m = (l + r) / 2;
		explore(node -> l, l, m + 0);
		explore(node -> r, m + 1, r);
	}
};

int n, q, a[MAXN + 5];
vector<int> occ[MAXN + 5];
PersistentTree pertree;

void init(int N, int Q, const std::vector<int> &input){
	pertree._n = n = N;
	pertree.build();
	KthSegmentTree segtree(N);
	for(int i = 0; i < n; i ++){
		a[i + 1] = input[i];
	}
	segtree.build();
	vector<pair<int, int>> intervals;
	for(int i = 1; i <= n; i ++){
		int v = a[i], sz = occ[v].size();
		if(0 < sz && sz < v){
			segtree.update(1, occ[v][sz - 1], -1);
		}
		else if(sz >= v){
			if(sz > v){
				segtree.update(1, occ[v][sz - (v + 1)], -1);
			}
			segtree.update(occ[v][sz - v] + 1, occ[v][sz - 1], -1);
		}
		occ[v].push_back(i); ++ sz;
		if(0 < sz && sz < v){
			segtree.update(1, occ[v][sz - 1], +1);
		}
		else if(sz >= v){
			if(sz > v){
				segtree.update(1, occ[v][sz - (v + 1)], +1);
			}
			segtree.update(occ[v][sz - v] + 1, occ[v][sz - 1], +1);
		}
		auto temp = segtree.query(1, i);
		if(temp.val_min == 0){
			for(int kth = 1; kth <= temp.cnt_min; kth ++){
				int l = segtree.descend(kth);
				intervals.push_back({l, i});
			}
		}
	}
	sort(intervals.begin(), intervals.end());
	int intervals_size = intervals.size();
	for(int i = 0; i < intervals_size; ){
		int j = i;
		while(j < intervals_size && intervals[i].first == intervals[j].first) ++ j;
		int old_ver = (i == 0 ? 0 : intervals[i - 1].first);
		int new_ver = intervals[i].first;
		for(int k = i; k < j; k ++){
			pertree.update(old_ver, new_ver, intervals[k].second, +1);
			old_ver = new_ver;
		}
		i = j;
	}
	for(int t = 1; t <= n; t ++){
		if(pertree.roots[t] == nullptr){
			pertree.roots[t] = pertree.roots[t - 1];
		}
	}
}

long long query(int l, int r){ ++ l; ++ r;
	return pertree.query(n, l, r) - pertree.query(l - 1, l, r);
}
