#!/bin/bash

author="author"
task_name="test"

# Read range from command-line arguments
a=$1
b=$2

# Check if a and b are provided
if [[ -z "$a" || -z "$b" ]]; then
    echo "Usage: $0 <start_number> <end_number>"
    exit 1
fi

# Check if a and b are integers
if ! [[ "$a" =~ ^[0-9]+$ && "$b" =~ ^[0-9]+$ ]]; then
    echo "Error: start_number and end_number must be integers."
    exit 1
fi

# Compile the program
g++ -o "$author" testing/Lgrader.cpp testing/author.cpp -O2 || { echo "Compilation failed"; exit 1; }

# Loop over the specified range
for ((i=a; i<=b; i++)); do
    file_number=$(printf "%03d" "$i")
    input_file="./${task_name}.${file_number}.in"
    
    if [[ -f "$input_file" ]]; then
        echo "Processing $input_file"
        solution_file="${task_name}.${file_number}.out"
        ./"$author" < "$input_file" | tee "$solution_file"
        echo "Created solution file: $solution_file"
    else
        echo "Input file not found: $input_file (skipping)"
    fi
done

